#include "HardwareProfile.h"
#include "GenericTypeDefs.h"
#include "Compiler.h"
#include "USB/usb_function_cdc.h"


#include "comm.h"
#include "dmx.h"
#include <delays.h>


#pragma udata
unsigned char CDCBufferOut[CDC_DATA_OUT_EP_SIZE];
unsigned char CDCBufferIn[CDC_DATA_IN_EP_SIZE];
unsigned char green_led_counter;

unsigned char txmiss_count = 0;


void comm_send_rply(unsigned char cc) {
	CDCBufferOut[CMD_HD_SYNC]	= CMD_SYNC_RECV;
	CDCBufferOut[CMD_HD_COMMAND]	= cc;
	CDCBufferOut[CMD_HD_LEN]	= 0x00;
	CDCBufferOut[CMD_HD_CHECK] 	= CDCBufferOut[CMD_HD_SYNC] ^ CDCBufferOut[CMD_HD_COMMAND] ^ CDCBufferOut[CMD_HD_LEN];

	if(USBUSARTIsTxTrfReady()) {
		putUSBUSART(CDCBufferOut,4);
	} else {
		txmiss_count++;
	}
}

void comm_send_dat(unsigned char len) {
	unsigned char i;
	if (len >60) {len = 60;}
	
	CDCBufferOut[CMD_HD_SYNC]	= CMD_SYNC_RECV;
	CDCBufferOut[CMD_HD_COMMAND]	= CMD_SYS_DATA;
	CDCBufferOut[CMD_HD_LEN]	= len;
	CDCBufferOut[CMD_HD_CHECK] 	= CDCBufferOut[CMD_HD_SYNC] ^ CDCBufferOut[CMD_HD_COMMAND] ^ CDCBufferOut[CMD_HD_LEN];

	len+=4;
	// calc checksum
	for (i = CMD_HD_LEN ; i<len; i++) {
		CDCBufferOut[CMD_HD_CHECK] ^= CDCBufferOut[i];
	}
	
	if(USBUSARTIsTxTrfReady()) {
		putUSBUSART(CDCBufferOut,len);
	} else {
		txmiss_count++;
	}
}



void ProcessCDC(void)
{   
    unsigned char numInputBytes, len, cmd, csum;
    int _x, dmx_pointer;

    unsigned char checksum;
    unsigned char boot_request0 = 0x23;
    unsigned char boot_request1 = 0x42;

    // User Application USB tasks
    if ((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1)) return;

    // check for input
    numInputBytes = getsUSBUSART(CDCBufferIn,CDC_DATA_IN_EP_SIZE); //until the buffer is free.
            
    if ((numInputBytes >= CMD_DATA_START) && (CDCBufferIn[CMD_HD_SYNC] == CMD_SYNC_SEND)) {
        len  = CDCBufferIn[CMD_HD_LEN];
        cmd  = CDCBufferIn[CMD_HD_COMMAND];
        csum = CDCBufferIn[CMD_HD_CHECK];
    } else {
        // not enough bytes received
        return;
    }
    
    // check for vaild length
    if ((len + CMD_DATA_START) != numInputBytes) {
        comm_send_rply(CMD_SYS_NACK_LENGTH);
        return;
    }
	
    // calculate checksum...
    checksum= CDCBufferIn[CMD_HD_SYNC] ^ CDCBufferIn[CMD_HD_COMMAND];
    for (_x = CMD_HD_LEN; _x < numInputBytes; _x++) {
            checksum ^= CDCBufferIn[_x];
    }
/*    if (csum != checksum) {
        comm_send_rply(CMD_SYS_NACK_CHECK);
        return;
    }*/

    // blink green led
    mLED_2_On();
    green_led_counter=128;

    // process command
    switch (cmd) {
        case CMD_SET_DATA_00:
        case CMD_SET_DATA_01:
        case CMD_SET_DATA_02:
        case CMD_SET_DATA_03:
        case CMD_SET_DATA_04:
        case CMD_SET_DATA_05:
        case CMD_SET_DATA_06:
        case CMD_SET_DATA_07:
            if (len == 32) {
                dmx_pointer = (cmd - CMD_SET_DATA_00) * 32;
                for (_x = CMD_DATA_START; _x < numInputBytes; _x++) {
                    dmx_buffer_0[dmx_pointer++] = CDCBufferIn[_x];
                }
                comm_send_rply(CMD_SYS_ACK);
            } else {
                comm_send_rply(CMD_SYS_NACK_LENGTH);
            }
            break;

        case CMD_SET_DATA_08:
        case CMD_SET_DATA_09:
        case CMD_SET_DATA_10:
        case CMD_SET_DATA_11:
        case CMD_SET_DATA_12:
        case CMD_SET_DATA_13:
        case CMD_SET_DATA_14:
        case CMD_SET_DATA_15:
            if (len == 32) {
                dmx_pointer = (cmd - CMD_SET_DATA_08) * 32;
                for (_x = CMD_DATA_START; _x < numInputBytes; _x++) {
                    dmx_buffer_1[dmx_pointer++] = CDCBufferIn[_x];
                }
                comm_send_rply(CMD_SYS_ACK);
            } else {
                comm_send_rply(CMD_SYS_NACK_LENGTH);
            }
            break;

        case CMD_GET_N_CHANNELS:
            if (len == 0) {
                    CDCBufferOut[CMD_DATA_START + 0] = 0; // low byte
                    CDCBufferOut[CMD_DATA_START + 1] = 2; //high byte
                    comm_send_dat(2);
            } else {
                    comm_send_rply(CMD_SYS_NACK_LENGTH);
            }
            break;

        case CMD_GET_VERSION:
            if (len == 0) {
                    CDCBufferOut[CMD_DATA_START] = FW_VERSION;
                    comm_send_dat(1);
            } else {
                    comm_send_rply(CMD_SYS_NACK_LENGTH);
            }
            break;
		    	
        case CMD_GET_HARDWARE:
            if (len == 0) {
                    CDCBufferOut[CMD_DATA_START] = HW_ID_USB2DMX;
                    comm_send_dat(1);
            } else {
                    comm_send_rply(CMD_SYS_NACK_LENGTH);
            }
            break;

        case CMD_BOOT_REQUEST:
            if (len == 0) {
                    CDCBufferOut[CMD_DATA_START + 0] = boot_request0;
                    CDCBufferOut[CMD_DATA_START + 1] = boot_request1;
                    comm_send_dat(2);
            } else {
                    comm_send_rply(CMD_SYS_NACK_LENGTH);
            }
            break;

        case CMD_BOOT_START:
            if (len == 2) {
                    if ((CDCBufferIn[CMD_DATA_START] == boot_request0) && (CDCBufferIn[CMD_DATA_START + 1] == boot_request1)) {
                            INTCONbits.GIE = 0;              // shut down global interrupts
                                    USBSoftDetach();
                                    USBDeviceTasks();
                                    Delay10TCYx(18); // 15ms
                                    _asm goto BOOTLOAD_START _endasm;
                            } else {
                                    // request did not match..
                                    comm_send_rply(CMD_SYS_NACK);
                            }
            } else {
                    comm_send_rply(CMD_SYS_NACK_LENGTH);
            }
            break;



        default:
            comm_send_rply(CMD_SYS_NIMP);
            break;
        } // of switch(cmd)
    
}//end ProcessIO
