#include "irdecoders.h"
#include <p18cxxx.h>
#include <timers.h>
#include "HardwareProfile.h"

enum TYPE_IR_STATE ir_state = IR_RESET_STATE;

//enum TYPE_IR_STATE l0_ir_state = IR_RESET_STATE;
//enum TYPE_IR_STATE l1_ir_state = IR_RESET_STATE;
//static unsigned int		l0_timediff = 0;
//static unsigned int		l1_timediff = 0;

unsigned long ir_receive_buffer = 0;
unsigned char ir_bitcount = 0;
unsigned char ir_toggle = 2;

unsigned char ir_proto = 0;
unsigned int  ir_command = 0;
unsigned int  ir_address = 0;
unsigned char ir_repeat = 0;
unsigned char ir_flags  = 0;


static unsigned char	ir_help_counter;
static unsigned int		timediff = 0;
unsigned char 			ir_pin_state;


void emit_one(void) {
	ir_receive_buffer = ir_receive_buffer << 1;
	ir_receive_buffer |= 0x01;							// emit one
	ir_bitcount++;
}
void emit_zero(void) {
	ir_receive_buffer = ir_receive_buffer << 1;
	ir_bitcount++;
}


void IR_Decode(void) {
	// calculate pulse/pause duration
	timediff = ReadTimer0();
	WriteTimer0(0);

	// read pin status
	ir_pin_state = !PORTBbits.RB4;
	
	if (IRIntFlag) {
		IRIntFlag=0;

/*		l1_ir_state = l0_ir_state;
		l1_timediff = l0_timediff;
		l0_ir_state = ir_state;
		l0_timediff = timediff;*/
		switch (ir_state) {
			
			// new pulse received
			case IR_RECV_OK:
			case IR_RESET_STATE:
				ir_bitcount = 0;
				if (ir_pin_state) {
					ir_proto = 0;
					ir_repeat = 0;
					ir_state = IR_END_OF_FIRST_HIGH;
				}
				break;
			
			// Branch!!
			case IR_END_OF_FIRST_HIGH:
				if ((timediff > IR_RC5_PULSE_SHORT_MIN) & (timediff < IR_RC5_PULSE_SHORT_MAX)) {
					// go for RC5
					ir_proto = IRP_RC5;
					ir_state = IR_RC5_DECODE;
					ir_bitcount = 1;				// startbit S1 already received
					ir_help_counter = 2;			// we are at the end of bit 1
					ir_receive_buffer = 0x00000001;	// bit1 = 1
				} else if ((timediff > IR_RC5_PULSE_LONG_MIN) & (timediff < IR_RC5_PULSE_LONG_MAX)) {
					// go for RC5-extended (startbit2 == 1)
					ir_proto = IRP_RC5;
					ir_state = IR_RC5_DECODE;
					ir_bitcount = 2;				// startbit S1 already received
					ir_help_counter = 3;			// we are at the mid of bit 2
					ir_receive_buffer = 0x00000002;	// bit1=1, bit2=0
				} else if ((timediff > IR_SHARP_PULSE_SHORT_MIN) & (timediff < IR_SHARP_PULSE_SHORT_MAX)) {
					ir_proto = IRP_SHARP;
					ir_state = IR_SHARP_CHECK_PAUSE;
					ir_receive_buffer = 0x00000000;
				} else if ((timediff > IR_NEC_AGC0_MIN) & (timediff < IR_NEC_AGC0_MAX)) {
					ir_proto = IRP_NEC;
					ir_state = IR_NEC_CHECK_AGC1;
					// ir_receive_buffer is reset in check-agc1-state
				} else if ((timediff > IR_RC6_LEADER_PULSE_MIN) & (timediff < IR_RC6_LEADER_PULSE_MAX)) {
					ir_proto = IRP_RC6;
					ir_state = IR_RC6_CHECK_LEADER_SPACE;
				} else {
					ir_state = IR_RESET_STATE;
				}
				break;
			/*
				**************** RC6-Decoder ****************
			*/
			case IR_RC6_CHECK_LEADER_SPACE:
				if ((timediff > IR_RC6_LEADER_SPACE_MIN) & (timediff <  IR_RC6_LEADER_SPACE_MAX)) {
					ir_state = IR_RC6_DECODE;
					ir_bitcount = 1;			
					ir_help_counter = 0;	
					ir_receive_buffer = 0x00000001; // this is actually a start
				} else {
					ir_state = IR_RESET_STATE;
				}			
				break;

			case IR_RC6_DECODE:
				switch (ir_bitcount) {
					case IR_RC6_BIT_TOGGLE_POS:
						// deceide if this is RC6A
						if (ir_receive_buffer == 0x1E) {
							ir_proto = IRP_RC6A;	
						}
						
						// update status
						if ((timediff > IR_RC6_PULSE_MIN) & (timediff < IR_RC6_PULSE_MAX))
						{
							ir_help_counter++;
						}
						else if ((timediff > IR_RC6_PULSE_2X_MIN) & (timediff < IR_RC6_PULSE_2X_MAX))
						{
							ir_help_counter++;
						}
						else if ((timediff > IR_RC6_PULSE_3X_MIN) & (timediff < IR_RC6_PULSE_3X_MAX))
						{
							ir_help_counter++;
							ir_help_counter++;
						}
						else {
							ir_state = IR_RESET_STATE;
						}
						break;
						
					case (IR_RC6_BIT_TOGGLE_POS+1):
						if ((ir_proto == IRP_RC6) &((ir_receive_buffer & 0x01) != ir_toggle)) {	// store toggle-bit
							ir_toggle = (ir_receive_buffer & 0x01);
						} 
						
						if ((timediff > IR_RC6_PULSE_2X_MIN) & (timediff < IR_RC6_PULSE_2X_MAX))
						{
							ir_help_counter++;
						}
						else if ((timediff > IR_RC6_PULSE_3X_MIN) & (timediff < IR_RC6_PULSE_3X_MAX))
						{
							ir_help_counter++;
							ir_help_counter++;
						}
						else {
							ir_state = IR_RESET_STATE;
						}
						
						break;
					default:
						// regular bits..
						if ((timediff > IR_RC6_PULSE_MIN) & (timediff < IR_RC6_PULSE_MAX))
						{
							ir_help_counter++;
						}
						else if ((timediff > IR_RC6_PULSE_2X_MIN) & (timediff < IR_RC6_PULSE_2X_MAX))
						{
							ir_help_counter++;
							ir_help_counter++;
						}
						else {
							ir_state = IR_RESET_STATE;
						}
						break;
				} // of switch-bitcount
				
				// manchester-fu!
				// if we are in the middle of an bit, emmit it!
				if ((ir_help_counter & 0x01) == 0x01) {
					if (!ir_pin_state) {
						emit_one();
					} else {
						emit_zero();
					}
				}
				break;

			/*
				**************** SHARP-Decoder ****************
			*/
			case IR_SHARP_CHECK_PAUSE:
				if ((timediff > IR_SHARP_PAUSE_SHORT_MIN) & (timediff <  IR_SHARP_PAUSE_SHORT_MAX)) {
					emit_zero();
					ir_state = IR_SHARP_CHECK_PULSE;
				} else if ((timediff > IR_SHARP_PAUSE_LONG_MIN) & (timediff <  IR_SHARP_PAUSE_LONG_MAX))
				{
					emit_one();
					ir_state = IR_SHARP_CHECK_PULSE;
				} else {
					ir_state = IR_RESET_STATE;
				}
				break;
			case IR_SHARP_CHECK_PULSE:
				if ((timediff > IR_SHARP_PULSE_SHORT_MIN) & (timediff < IR_SHARP_PULSE_SHORT_MAX)) {
					// Pulse-length was okay
					ir_state = IR_SHARP_CHECK_PAUSE;
				} else {
					ir_state = IR_RESET_STATE;
				}
				break;

			/*
				**************** RC5-Decoder ****************
			*/
			case IR_RC5_DECODE:
				if ((timediff > IR_RC5_PULSE_SHORT_MIN) & (timediff < IR_RC5_PULSE_SHORT_MAX))
				{
					ir_help_counter++;
				}
				else if ((timediff > IR_RC5_PULSE_LONG_MIN) & (timediff < IR_RC5_PULSE_LONG_MAX))
				{
					ir_help_counter++;
					ir_help_counter++;
				}
				else {
					ir_state = IR_RESET_STATE;
				}
				// if we are in the middle of an bit, emmit it!
				if ((ir_help_counter & 0x01) == 0x01) {
					if (ir_pin_state) {
						emit_one();
					} else {
						emit_zero();
					}
				}
				break;
		
			
			/*
				**************** NEC-Decoder ****************
			*/
			case IR_NEC_CHECK_AGC1:
				if ((timediff > IR_NEC_AGC1_MIN) & (timediff < IR_NEC_AGC1_MAX)) {
					ir_receive_buffer = 0x00000000;
					ir_state = IR_NEC_CHECK_PULSE;
				} else if ((timediff > IR_NEC_REPEAT_MIN) & (timediff < IR_NEC_REPEAT_MAX)) {
					// this may be a repeat-signal
					ir_state = IR_NEC_CHECK_REPEAT;
				} else {
					ir_state = IR_RESET_STATE;
				}
				break;
			case IR_NEC_CHECK_PULSE:
				if ((timediff > IR_NEC_PULSE_MIN) & (timediff < IR_NEC_PULSE_MAX)) {
					ir_state = IR_NEC_CHECK_PAUSE;
				} else {
					ir_state = IR_RESET_STATE;
				}
				break;	
			case IR_NEC_CHECK_PAUSE:
				if ((timediff > IR_NEC_PULSE_MIN) & (timediff < IR_NEC_PULSE_MAX)) {
					ir_state = IR_NEC_CHECK_PULSE;
					emit_zero();
				} else if ((timediff > IR_NEC_PULSE_3X_MIN) & (timediff < IR_NEC_PULSE_3X_MAX)) {
					ir_state = IR_NEC_CHECK_PULSE;
					emit_one();
				} else {
					ir_state = IR_RESET_STATE;
				}
				break;	
			case IR_NEC_CHECK_REPEAT:
				if ((timediff > IR_NEC_PULSE_MIN) & (timediff < IR_NEC_PULSE_MAX)) {
					ir_state = IR_NEC_REPEAT_OK;
				} else {
					ir_state = IR_RESET_STATE;
				}
				break;
			

			// state-maschine is save... should not happen!
			default:
				ir_state = IR_FINAL_STATE;
				break;
			
		} // end of switch
		
		
		/*
				**************** BEGIN_DECODING Check ****************
		*/
		if ((ir_proto == IRP_RC6) & (ir_bitcount == 22)) {
			ir_state = IR_RECV_OK;
			ir_command = ir_receive_buffer & 0xFF; 			// 8-bits command
			ir_address = (ir_receive_buffer >> 8)& 0xFF;	// 8-bits address
			ir_flags |= IR_FLAG_DATA_OK;
		}
		
		if ((ir_proto == IRP_RC6A) & (ir_bitcount == 38)) {
			ir_state = IR_RECV_OK;
			ir_command = ir_receive_buffer & 0x7FFF;		// 15-bits command
			ir_address = (ir_receive_buffer >> 16)& 0xFFFF;	// 16-bits address

			// check toggle bit
			if ((ir_receive_buffer >> 15 & 0x01) == ir_toggle) {
				ir_repeat = 1;
			} 
			ir_toggle = (ir_receive_buffer >> 15 & 0x01);

			ir_flags |= IR_FLAG_DATA_OK;
		}
		
		if ((ir_proto == IRP_RC5) & (ir_bitcount == 14)) {
			ir_state = IR_RECV_OK;
			ir_command = ir_receive_buffer & 0x7F; 			// 7-bits command
			ir_address = (ir_receive_buffer >> 6)& 0x1F;	// 5-bits address
			
			//check field bit
			if ((ir_receive_buffer >> 12 & 0x01) == 0x0) {
				ir_command+=64;	// rc5-extended mode
			}
			// check toggle bit
			if ((ir_receive_buffer >> 11 & 0x01) == ir_toggle) {
				ir_repeat = 1;
			} 
			ir_toggle = (ir_receive_buffer >> 11 & 0x01);
			
			ir_flags |= IR_FLAG_DATA_OK;
		}
		
		if ((ir_proto == IRP_NEC) & (ir_bitcount == 0x20)) {
			ir_command = (~ir_receive_buffer) & 0xFF;		// 8-bits command
			// command is send twice (once inverted... check this)
			if (ir_command == ((ir_receive_buffer >> 8) & 0xFF)) {
				ir_state = IR_RECV_OK;
				ir_address = (ir_receive_buffer >> 16)& 0xFFFF;
				ir_repeat = 0;
				ir_flags |= IR_FLAG_DATA_OK;
			}
		}
		if ((ir_proto == IRP_NEC) & (ir_state == IR_NEC_REPEAT_OK)) {
			ir_state = IR_RECV_OK;
			ir_repeat = 1;
			ir_flags |= IR_FLAG_DATA_OK;
		}	
		
		
		if ((ir_proto == IRP_SHARP) & (ir_bitcount == 0x0F)) {
			ir_help_counter = (ir_receive_buffer & 0x03);
			if (ir_help_counter == 0x02) {
				ir_state = IR_RECV_OK;
				ir_command = (ir_receive_buffer >> 2)  & 0xFF;
				ir_address = (ir_receive_buffer >> 10) & 0x1F;
				ir_repeat = 0;
			}
		}

	} else if (INTCONbits.TMR0IF){
		// Timer0 berlauf
		INTCONbits.TMR0IF = 0;
		
		if ((ir_bitcount > 0) & (ir_state >= 0x10)) {
			ir_repeat = 22;
			ir_bitcount = 0;
		}
		ir_state = IR_RESET_STATE;
	} // ende zustandsmaschine
	
}